/* This is a script that lists the day's callers.
 *
 * Author: Eric Oulashin (AKA Nightfox)
 * BBS: Digital Distortion
 * BBS address: digdist.bbsindex.com
 *
 * Date       User              Description
 * 2009-04-30 Eric Oulashin     Created
 * 2009-05-01 Eric Oulashin     Continued work
 * 2009-05-02 Eric Oulashin     Continued work - Used substring to make
 *                              sure field values don't exceed the field
 *                              widths.  Updated to read logon.lst in the
 *                              data directory rather than going through
 *                              the user records and matching their last
 *                              logon date.
 * 2009-05-03 Eric Oulashin     Refactored the script a bit.  Added a
 *                              boolean, pauseAtEnd, which will control
 *                              whether or not the script will pause at
 *                              the end, and set up argv[0] to change
 *                              that too.  Also, added a Synchronet
 *                              version check.
 * 2009-05-04 Eric Oulashin     Changed the User variable's name from user
 *                              to theUser to avoid stomping on the user
 *                              object declared in sbbsdefs.js.  Also,
 *                              updated to read through the system user
 *                              records if unable to read the logon.lst file.
 *                              Either way, the caller information is now
 *                              placed in an array, and the last X callers
 *                              are displayed at the end.  Also, made use
 *                              of the newly-added variable DIGDIST_INCLUDE_SYSOP
 *                              in DigitalDistortionDefs.js.
 * 2009-05-09 Eric Oulashin     Updated to only add the caller information to
 *                              the array if the user's handle & logon time
 *                              read from logon.lst are not blank.  This fixed
 *                              a bug that was causing the script to occasionally
 *                              show a single line in the logon list for the day's
 *                              first caller.
 * 2009-05-10 Eric Oulashin     Removed the "donePrompt" color from the colors
 *                              array, because it wasn't being used.
 * 2009-05-13 Eric Oulashin     Updated to be free of DigitalDistortionDefs.js.
 *                              Added VERSION and VER_DATE, although they aren't
 *                              displayed to the user.
 */

load("sbbsdefs.js");
//load("DigitalDistortionDefs.js");

// This script requires Synchronet version 3.10 or higher.
// Exit if the Synchronet version is below the minimum.
if (system.version_num < 31000)
{
	var message = "\1n\1h\1y\1i* Warning:\1n\1h\1w Digital Distortion List Today's Callers "
	             + "requires version \1g3.10\1w or\r\n"
	             + "higher of Synchronet.  This BBS is using version \1g" + system.version
	             + "\1w.  Please notify the sysop.";
	console.crlf();
	console.print(message);
	console.crlf();
	console.print("\1p");
	exit();
}

// Colors
var colors = new Array();
colors["veryTopHeader"] = "\1n\1h\1w";
colors["border"] = "\1n\1c";
colors["colLabelText"] = "\1n4\1h\1w";
colors["timeOn"] = "\1h\1c";
colors["userName"] = "\1h\1y";
colors["location"] = "\1h\1m";
colors["nodeNumber"] = "\1h\1r";
colors["connectionType"] = "\1h\1b";
colors["numCalls"] = "\1h\1g";

// Program option variables
// pauseAtEnd controls whether or not to pause at the end of the listing.
var pauseAtEnd = true;
// The maximum number of callers to display
var MAX_CALLERS = 6;
// Whether or not to include the sysop in the list.
var includeSysop = true;

// Column inner lengths
var TIME_ON_LEN = 7;
var USER_NAME_LEN = 18;
var LOCATION_LEN = 25;
var NODE_NUM_LEN = 5;
var CONNECTION_TYPE_LEN = 10;
var NUM_CALLS_LEN = 6;

const VERSION = "1.01";
const VER_DATE = "2009-05-13";

// Single-line box drawing/border characters.
const UPPER_LEFT_SINGLE = "";
const HORIZONTAL_SINGLE = "";
const UPPER_RIGHT_SINGLE = "";
const VERTICAL_SINGLE = "";
const LOWER_LEFT_SINGLE = "";
const LOWER_RIGHT_SINGLE = "";
const T_SINGLE = "";
const LEFT_T_SINGLE = "";
const RIGHT_T_SINGLE = "";
const BOTTOM_T_SINGLE = "";
const CROSS_SINGLE = "";
// Double-line box drawing/border characters.
const UPPER_LEFT_DOUBLE = "";
const HORIZONTAL_DOUBLE = "";
const UPPER_RIGHT_DOUBLE = "";
const VERTICAL_DOUBLE = "";
const LOWER_LEFT_DOUBLE = "";
const LOWER_RIGHT_DOUBLE = "";
const T_DOUBLE = "";
const LEFT_T_DOUBLE = "";
const RIGHT_T_DOUBLE = "";
const BOTTOM_T_DOUBLE = "";
const CROSS_DOUBLE = "";

// Let the first program argument change pauseAtEnd.
if (typeof(argv[0]) != "undefined")
	pauseAtEnd = argv[0];
// Let the second program argument change MAX_CALLERS.
if (typeof(argv[1]) != "undefined")
	MAX_CALLERS = argv[1];


// This is the message that is displayed if the user is the first caller of the day.
var firstCallerMessage = colors["veryTopHeader"] + "You are the first caller of the day!";

// The following variable is used for a line of text throughout this script.
var lineText = "";


///////////////////////////
// Main script execution //
///////////////////////////

// Clear the screen
console.clear();

// The following variable will be set to true if the user is the first caller
// of the day.  This will control whether the day's caller list is shown or
// if the user will be told that they are the first caller of the day.
var firstCaller = (system.stats.logons_today == 0);

// If the user is not the first caller, then open logon.lst in the data dir.
// If the file can't be opened, then assume the user is the first caller of
// the day.
var loginFile = null;
var loginFileOpened = false;
if (!firstCaller)
{
	loginFile = new File(system.data_dir + "logon.lst");
	if (loginFile.exists)
		loginFileOpened = loginFile.open("r", true);
}

// Create the array that will contain the caller information
var callerInfoArray = new Array();

// See if the user is the first caller.
if (firstCaller)
{
	// Do nothing.  The first caller message will be displayed later.
}
// The user is not the first caller.  If the login file was opened, read the
// caller information from it.
else if (loginFileOpened && (loginFile != null))
{
	// Read the lines from the file and insert the caller information into
	// callerInfoArray.
	var fileLine = "";
	var userHandle = ""; // For testing for valid data
	var logonTime = "";  // For testing for valid data
	var callerInfo = null;
	while (!loginFile.eof)
	{
		// Read the line from the file, stripping control characters.
		fileLine = strip_ctrl(loginFile.readln(512));
	
		// Skip blank lines
		if (fileLine == "")
			continue;
		
		/* Fields
		   ------
		   0-2: Node #
		   3-9: Total # of calls to the system?
		   10-35: User handle
		   36-60: User location
		   61-65: Login time
		   67-74: Connection type
		   76-80: The user's number of calls today
		*/
		// Only add the caller info if the user's handle & logon time are
		// not blank.
		userHandle = trimSpaces(fileLine.substring(10, 36), false, false, true);
		userLogonTime = trimSpaces(fileLine.substring(61, 66), false, false, true);
		if ((userHandle != "") && (userLogonTime != ""))
		{
			callerInfo = new Object();
			callerInfo.nodeNumber = trimSpaces(fileLine.substring(0, 2), false, false, true);
			callerInfo.handle = userHandle;
			callerInfo.location = trimSpaces(fileLine.substring(36, 61), false, false, true);
			callerInfo.logonTime = userLogonTime;
			callerInfo.connectionType = trimSpaces(fileLine.substring(67, 75), false, false, true);
			callerInfo.numCallsToday = trimSpaces(fileLine.substring(76, 81), false, false, true);
			callerInfoArray.push(callerInfo);
		}
	}
	loginFile.close();
}
// The user is not the first caller, and the login file was not opened.  Look through
// the user records for users who called today.
else
{
	// Go through each user record.  If their last logon date was today, then
	// add their user information to callerInfoArray.
	var callerInfo = null;
	for (var userNum = 1; userNum <= system.stats.total_users; ++userNum)
	{
		// If the sysop is not to be included, then skip user # 1 (user #1
		// is more than likely the sysop).
		if (!includeSysop && (userNum == 1))
			continue;

		theUser = new User(userNum);
		// If the user logged on today, then add the user information to
		// callerInfoArray.
		if (theUser.stats.logons_today > 0)
		{
			callerInfo = new Object();
			callerInfo.nodeNumber = "";
			callerInfo.handle = theUser.alias;
			callerInfo.location = theUser.location;
			callerInfo.logonTime = strftime("%H:%M", theUser.stats.laston_date);
			callerInfo.connectionType = theUser.connection;
			callerInfo.numCallsToday = theUser.stats.logons_today;
			callerInfoArray.push(callerInfo);
		}
	}

	// Sort the array by user logon time
	callerInfoArray.sort(function(pA, pB)
	{
		// Get both users' logon times, converted to numbers without the colons.
		var userALogonTime = +pA.logonTime.replace(":", "");
		var userBLogonTime = +pB.logonTime.replace(":", "");
		
		// Return -1, 0, or 1, depending on whether user A's logon
		// time is before, equal, or after user B's logon time.
		var returnValue = 0;
		if (userALogonTime < userBLogonTime)
			returnValue = -1;
		else if (userALogonTime > userBLogonTime)
			returnValue = 1;
			
		return returnValue;
	});
}

// If callerInfoArray has any caller information, then display it.
if (callerInfoArray.length > 0)
{
	// The following line which sets console.line_counter to 0 is a
	// kludge to disable Synchronet's automatic pausing after a
	// screenful of text.
	console.line_counter = 0;
	
	// Write the caller list header
	writeHeader();
	
	// We want to display the last MAX_CALLERS number of callers.  First,
	// calculate the starting array index based on MAX_CALLERS.  If
	// MAX_CALLERS is 0 or negative, then display all the callers in the
	// array.
	var startIndex = 0;
	if ((MAX_CALLERS > 0) && (callerInfoArray.length > MAX_CALLERS))
		startIndex = callerInfoArray.length - MAX_CALLERS;
	// Display the callers.
	for (var i = startIndex; i < callerInfoArray.length; ++i)
	{
		lineText = colors["border"] + VERTICAL_SINGLE;
		lineText += colors["timeOn"] + centerText(callerInfoArray[i].logonTime, TIME_ON_LEN);
		lineText += colors["border"] + VERTICAL_SINGLE;
		lineText += colors["userName"] + centerText(callerInfoArray[i].handle.substring(0, USER_NAME_LEN), USER_NAME_LEN);
		lineText += colors["border"] + VERTICAL_SINGLE;
		lineText += colors["location"] + centerText(callerInfoArray[i].location.substring(0, LOCATION_LEN), LOCATION_LEN);
		lineText += colors["border"] + VERTICAL_SINGLE;
		lineText += colors["nodeNumber"] + centerText(callerInfoArray[i].nodeNumber, NODE_NUM_LEN)
		lineText += colors["border"] + VERTICAL_SINGLE;
		lineText += colors["connectionType"];
		lineText += centerText(callerInfoArray[i].connectionType.substring(0, CONNECTION_TYPE_LEN), CONNECTION_TYPE_LEN);
		lineText += colors["border"] + VERTICAL_SINGLE;
		lineText += colors["numCalls"];
		lineText += centerText(format("%" + NUM_CALLS_LEN + "d", callerInfoArray[i].numCallsToday), NUM_CALLS_LEN);
		lineText += colors["border"] + VERTICAL_SINGLE;
		console.center(lineText);
	}
	
	// Write the end border for the table
	writeEndBorder();
}
else
{
	// There are no elements in callerInfoArray, so tell the user that they
	// are the first caller.
	console.center(firstCallerMessage);
}

// We're done.  If pauseAtEnd is true, then display the system's pause/"Hit a key" prompt.
if (pauseAtEnd)
	console.print("\1p");
	
// End of script execution

///////////////////////////////////////////////////////////////////////////////////
// Functions

// Writes the header for the user listing
function writeHeader()
{
	// Write the header
	console.print(colors["veryTopHeader"]);
	console.center("Last several callers");
	// Upper border lines
	// "Time on" section
	lineText = colors["border"] + UPPER_LEFT_SINGLE;
	for (var i = 0; i < TIME_ON_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// User name section
	for (var i = 0; i < USER_NAME_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// Location section
	for (var i = 0; i < LOCATION_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// Node number section
	for (var i = 0; i < NODE_NUM_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// Connection type section
	for (var i = 0; i < CONNECTION_TYPE_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// # of calls section
	for (var i = 0; i < NUM_CALLS_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += UPPER_RIGHT_SINGLE;
	console.center(lineText + "\r\n");

	// Column labels
	lineText = colors["border"] + VERTICAL_SINGLE + colors["colLabelText"]
			 + centerText("Time On", TIME_ON_LEN) + colors["border"] + VERTICAL_SINGLE
			 + colors["colLabelText"] + centerText("User Name", USER_NAME_LEN) + colors["border"]
			 + VERTICAL_SINGLE + colors["colLabelText"]
			 + centerText("Location", LOCATION_LEN) + colors["border"] + VERTICAL_SINGLE
			 + colors["colLabelText"]
			 + centerText("Node#", NODE_NUM_LEN)  + colors["border"] + VERTICAL_SINGLE
			 + colors["colLabelText"] + centerText("Connection", CONNECTION_TYPE_LEN)
			 + colors["border"] + VERTICAL_SINGLE + colors["colLabelText"]
			 + centerText("#Calls", NUM_CALLS_LEN) + colors["border"]
			 + VERTICAL_SINGLE;
	console.center(lineText + "\r\n");

	// Lower border lines for header
	// "Time on" section
	lineText = colors["border"] + LEFT_T_SINGLE;
	for (var i = 0; i < TIME_ON_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// User name section
	for (var i = 0; i < USER_NAME_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// Location section
	for (var i = 0; i < LOCATION_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// Node number section
	for (var i = 0; i < NODE_NUM_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// Connection type section
	for (var i = 0; i < CONNECTION_TYPE_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// # of calls section
	for (var i = 0; i < NUM_CALLS_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += RIGHT_T_SINGLE;
	console.center(lineText);
}

// Writes the end border
function writeEndBorder()
{
	// "Time on" section
	lineText = colors["border"] + LOWER_LEFT_SINGLE;
	for (var i = 0; i < TIME_ON_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// User name section
	for (var i = 0; i < USER_NAME_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// Location section
	for (var i = 0; i < LOCATION_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// Node number section
	for (var i = 0; i < NODE_NUM_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// Connection type section
	for (var i = 0; i < CONNECTION_TYPE_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// # of calls section
	for (var i = 0; i < NUM_CALLS_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += LOWER_RIGHT_SINGLE;
	console.center(lineText);
}

// Removes multiple, leading, and/or trailing spaces
// The search & replace regular expressions used in this
// function came from the following URL:
//  http://qodo.co.uk/blog/javascript-trim-leading-and-trailing-spaces
//
// Parameters:
//  pString: The string to trim
//  pLeading: Whether or not to trim leading spaces (optional, defaults to true)
//  pMultiple: Whether or not to trim multiple spaces (optional, defaults to true)
//  pTrailing: Whether or not to trim trailing spaces (optional, defaults to true)
function trimSpaces(pString, pLeading, pMultiple, pTrailing)
{
	var leading = true;
	var multiple = true;
	var trailing = true;
	if(typeof(pLeading) != "undefined")
		leading = pLeading;
	if(typeof(pMultiple) != "undefined")
		multiple = pMultiple;
	if(typeof(pTrailing) != "undefined")
		trailing = pTrailing;
		
	// To remove both leading & trailing spaces:
	//pString = pString.replace(/(^\s*)|(\s*$)/gi,"");

	if (leading)
		pString = pString.replace(/(^\s*)/gi,"");
	if (multiple)
		pString = pString.replace(/[ ]{2,}/gi," ");
	if (trailing)
		pString = pString.replace(/(\s*$)/gi,"");

	return pString;
}

// Centers some text within a specified field length.
//
// Parameters:
//  pText: The text to center
//  pFieldLen: The field length
function centerText(pText, pFieldLen)
{
	var centeredText = "";
	
	// If pFieldLen is less than the text length, then truncate the string.
	if (pFieldLen < pText.length)
	{
		centeredText = pText.substring(0, pFieldLen);
	}
	else
	{
		// pFieldLen is at least equal to the text length, so we can
		// center the text.
		// Calculate the number of spaces needed to center the text.
		var numSpaces = pFieldLen - pText.length;
		if (numSpaces > 0)
		{
			var rightSpaces = (numSpaces/2).toFixed(0);
			var leftSpaces = numSpaces - rightSpaces;
			// Build centeredText
			for (var i = 0; i < leftSpaces; ++i)
				centeredText += " ";
			centeredText += pText;
			for (var i = 0; i < rightSpaces; ++i)
				centeredText += " ";
		}
		else
		{
			// pFieldLength is the same length as the text.
			centeredText = pText;
		}
	}

	return centeredText;
}